import java.util.Random;

/**
 * A class to test a Bag.
 * 
 * @author Gloss
 * @version $Id: BagRandomTest.java 1767 2013-06-11 16:39:22 asantos $
 */
public class BagRandomTest {

    /**
     * Our random object to generate random int.
     */
    private Random rand;

    /**
     * Bag object.
     */
    private Bag<Object> bag;

    /**
     * How many tests do we want to perform.
     */
    private int howMany;

    /**
     * Max operations.
     */
    private final static int MAX_OPERATIONS = 5;
    
    /**
     * Max integers.
     */
    private final static int MAX_INTEGER = 11;

    /**
     * Constructor.
     * 
     * @param bag Bag object to be tested
     * @param howMany How many tests we want to perform.
     */
    public BagRandomTest(Bag<Object> bag, int howMany) {
        this.bag = bag;
        rand = new Random();
        this.howMany = howMany;
    }

    /**
     * Creates a new random integer.
     * 
     * @return The newly created integer.
     */
    private Integer newObject() {
        return new Integer(rand.nextInt(MAX_INTEGER));
    }

    /**
     * Runs the chosen number of tests/operations
     */
    public void run() {
        Object item = null;

        for (int i = 0; i < howMany; i++) {
        	
            switch (rand.nextInt(MAX_OPERATIONS)) {
            
                case 0: {
                	item = newObject();
                    System.out.print("add(" + item + ")");
                    bag.add(item);
                    break;
                }
                case 1: {
                	item = newObject();
                    System.out.print("contains(" + item + ")=");
                    System.out.print(bag.contains(item));
                    break;
                }
                case 2: {
                	item = newObject();
                    System.out.print("remove(" + item + ")=");
                    bag.remove(item);
                    break;
                }
                case 3: {
                	item = newObject();
                    System.out.print("howMany(" + item + ")=");
                    System.out.print(bag.howMany(item));
                    break;
                }
                case 4: {
                    System.out.print("isEmpty()=");
                    System.out.print(bag.isEmpty());
                    break;
                }
            }
            System.out.println();
        }
    }
}
